/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.geometry.model;

import java.util.ArrayList;
import java.util.List;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ui.OpenProjects;
import org.netbeans.spi.project.SubprojectProvider;
import org.openide.loaders.DataObject;

/**
 * Classes implementing this interface can provide Register implementations
 * using @ServiceProvider(service = FieldContainer.class)
 *
 * @author Gerben Feenstra & Merijn Zengers
 */
public interface GeometryContainer<T extends Shape> {
    
    List<T> getShapes();
    
    String getDisplayName();   
    
    /**
     * Import list of shapes and make field from them
     */
    void importShapes(List<Shape> shapes);

    /**
     * Source dataobject to import
     *
     * @param dataObject Source dataobject to import
     */
    void importSources(DataObject dataObject);

    Shape.ShapeType getShapeType();

    /**
     * If the geometry Container can handle this type of shape
     *
     * @param shapeType the shapeType to test
     * @return if the geometry container can handle this type of shape
     */
    boolean supports(Shape.ShapeType shapeType);

    static class Util {

        public static List<GeometryContainer> findGeometryContainerForShapeType(Shape.ShapeType shapeType) {
            List<GeometryContainer> fieldContainers = new ArrayList<>();

            for(GeometryContainer container : findOpenGeometryContainers()){
                if(container.supports(shapeType)){
                    fieldContainers.add(container);
                }
            }
            return fieldContainers;
        }

        /**
         * Helper method to find all geometry containers present
         * Only finds geometry containers with GeometryContainer registered in the lookup
         * @return
         */
        public static List<GeometryContainer> findOpenGeometryContainers() {
            List<GeometryContainer> fieldContainers = new ArrayList<>();

            // Add toplevel Fieldcontainers
            for (Project project : OpenProjects.getDefault().getOpenProjects()) {
                GeometryContainer topLevelContainer = project.getLookup().lookup(GeometryContainer.class);
                if (topLevelContainer != null) {
                    fieldContainers.add(topLevelContainer);
                }

                // Add subproject fieldcontainers
                SubprojectProvider subprojectContainer = project.getLookup().lookup(SubprojectProvider.class);
                if (subprojectContainer != null) {
                    for (Project subProject : subprojectContainer.getSubprojects()) {
                        GeometryContainer subProvider = subProject.getLookup().lookup(GeometryContainer.class);
                        if (subProvider != null) {
                            fieldContainers.add(subProvider);
                        }
                    }
                }
            }
            return fieldContainers;
        }
    }
}
